import 'dart:convert';
import 'dart:io';

import 'package:wings_iconfont/model/iconfont_model.dart';
import 'package:wings_iconfont/temp/iconfont_temp.dart';
import 'package:yaml/yaml.dart';

import 'model/yaml_model.dart';

String yamlPath = 'pubspec.yaml';

String firstUpper(String str) {
  var strArr = str.split('');
  strArr[0] = strArr[0].toUpperCase();
  return strArr.join();
}

String readFile(String path) {
  var file = File(path);
  return file.readAsStringSync();
}

void writeFile(String path, String contents) async {
  var file = File(path);
  bool exists = await file.exists();
  if (!exists) {
    file.createSync(recursive: true);
  }
  file.writeAsStringSync(contents);
}

YamlModel getYamlModel({String source}) {
  source = source ?? readYaml();
  return YamlModel.fromJson(jsonDecode(jsonEncode(loadYaml(source))));
}

String readYaml() {
  return readFile(yamlPath);
}

void writeYaml(String contents) {
  writeFile(yamlPath, contents);
}

void addYamlFont(String family, String path) {
  YamlModel yamlModel = getYamlModel();
  var yamlList = readYaml().split('\n');
  if (yamlModel?.flutter?.fonts == null) {
    var index = yamlList.indexWhere((e) => e == 'flutter:');
    yamlList.insert(index + 1, '''  fonts:
    - family: $family
      fonts:
        - asset: $path''');
    writeYaml(yamlList.join('\n'));
    yamlModel = getYamlModel(source: yamlList.join('\n'));
  }

  if (yamlModel.flutter.fonts.where((e) => e.family == family).length == 0) {
    var index = yamlList.indexWhere((e) => e == '  fonts:');
    yamlList.insert(index + 1, '''    - family: $family
      fonts:
        - asset: $path''');
    writeYaml(yamlList.join('\n'));
    yamlModel = getYamlModel(source: yamlList.join('\n'));
  }

  if (yamlModel.flutter.fonts.where((e) => e.family == family).where((e) => e.fonts.where((e1) => e1.asset.indexOf('$path') > -1).length > 0).length == 0) {
    var index = yamlList.indexWhere((e) => e == '    - family: $family');
    yamlList.insert(index + 2, '''        - asset: $path''');
    writeYaml(yamlList.join('\n'));
    yamlModel = getYamlModel(source: yamlList.join('\n'));
  }
}

void saveFont(String ttfPath, String jsonPath, String savePath, String clazzName) {
  var iconFontModel = IconFontModel.fromJson(json.decode(readFile(jsonPath)));

  String tmp = IconFontTemp.getIconFontTemp(iconFontModel, firstUpper(clazzName));
  addYamlFont(iconFontModel.fontFamily, ttfPath);

  writeFile(savePath, tmp);
}
